# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: olympicslocationagent.proto
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1bolympicslocationagent.proto\x12\x18olympicslocationagent.v1"-\n\x0c\x43ityLocation\x12\x0c\n\x04\x63ity\x18\x01 \x01(\t\x12\x0f\n\x07\x63ountry\x18\x02 \x01(\t"\x18\n\x08Olympics\x12\x0c\n\x04year\x18\x01 \x01(\x05\x32j\n\x15OlympicsLocationAgent\x12Q\n\x03\x41sk\x12".olympicslocationagent.v1.Olympics\x1a&.olympicslocationagent.v1.CityLocationb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "olympicslocationagent_pb2", _globals
)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_CITYLOCATION"]._serialized_start = 57
    _globals["_CITYLOCATION"]._serialized_end = 102
    _globals["_OLYMPICS"]._serialized_start = 104
    _globals["_OLYMPICS"]._serialized_end = 128
    _globals["_OLYMPICSLOCATIONAGENT"]._serialized_start = 130
    _globals["_OLYMPICSLOCATIONAGENT"]._serialized_end = 236
# @@protoc_insertion_point(module_scope)
