# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: greeter.proto
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\rgreeter.proto\x12\ngreeter.v1"\x1d\n\nHelloReply\x12\x0f\n\x07message\x18\x01 \x01(\t"\x1c\n\x0cHelloRequest\x12\x0c\n\x04name\x18\x01 \x01(\t2G\n\x07Greeter\x12<\n\x08SayHello\x12\x18.greeter.v1.HelloRequest\x1a\x16.greeter.v1.HelloReplyb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "greeter_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_HELLOREPLY"]._serialized_start = 29
    _globals["_HELLOREPLY"]._serialized_end = 58
    _globals["_HELLOREQUEST"]._serialized_start = 60
    _globals["_HELLOREQUEST"]._serialized_end = 88
    _globals["_GREETER"]._serialized_start = 90
    _globals["_GREETER"]._serialized_end = 161
# @@protoc_insertion_point(module_scope)
