# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fooservice.proto
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x10\x66ooservice.proto\x12\x06\x66oo.v1"y\n\x0b\x46ooResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03\x61ge\x18\x02 \x01(\x05\x12%\n\x01\x64\x18\x03 \x03(\x0b\x32\x1a.foo.v1.FooResponse.DEntry\x1a(\n\x06\x44\x45ntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"w\n\nFooRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03\x61ge\x18\x02 \x01(\x05\x12$\n\x01\x64\x18\x03 \x03(\x0b\x32\x19.foo.v1.FooRequest.DEntry\x1a(\n\x06\x44\x45ntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x32<\n\nFooService\x12.\n\x03\x46oo\x12\x12.foo.v1.FooRequest\x1a\x13.foo.v1.FooResponseb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "fooservice_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _FOORESPONSE_DENTRY._options = None
    _FOORESPONSE_DENTRY._serialized_options = b"8\001"
    _FOOREQUEST_DENTRY._options = None
    _FOOREQUEST_DENTRY._serialized_options = b"8\001"
    _globals["_FOORESPONSE"]._serialized_start = 28
    _globals["_FOORESPONSE"]._serialized_end = 149
    _globals["_FOORESPONSE_DENTRY"]._serialized_start = 109
    _globals["_FOORESPONSE_DENTRY"]._serialized_end = 149
    _globals["_FOOREQUEST"]._serialized_start = 151
    _globals["_FOOREQUEST"]._serialized_end = 270
    _globals["_FOOREQUEST_DENTRY"]._serialized_start = 109
    _globals["_FOOREQUEST_DENTRY"]._serialized_end = 149
    _globals["_FOOSERVICE"]._serialized_start = 272
    _globals["_FOOSERVICE"]._serialized_end = 332
# @@protoc_insertion_point(module_scope)
