# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: barservice.proto
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x10\x62\x61rservice.proto\x12\x06\x62\x61r.v1"\x1c\n\x0b\x42\x61rResponse\x12\r\n\x05names\x18\x01 \x03(\t"\x1b\n\nBarRequest\x12\r\n\x05names\x18\x01 \x03(\t2<\n\nBarService\x12.\n\x03\x42\x61r\x12\x12.bar.v1.BarRequest\x1a\x13.bar.v1.BarResponseb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "barservice_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_BARRESPONSE"]._serialized_start = 28
    _globals["_BARRESPONSE"]._serialized_end = 56
    _globals["_BARREQUEST"]._serialized_start = 58
    _globals["_BARREQUEST"]._serialized_end = 85
    _globals["_BARSERVICE"]._serialized_start = 87
    _globals["_BARSERVICE"]._serialized_end = 147
# @@protoc_insertion_point(module_scope)
